from setuptools import setup

setup(name='nonebot-plugin-gamedraw',
      version='0.1.7.7',
      description='Nonebot2插件',
      url='https://github.com/HibiKier/nonebot_plugin_gamedraw',
      author='hibikier',
      author_email='775757368@qq.com',
      license='MIT',
      packages=['nonebot_plugin_gamedraw'])
